from __future__ import annotations
from grpc import Channel

from typing import List, Dict, cast, Tuple

from ..protogen.client.results_service_pb2_grpc import ResultsStub
from ..protogen.common.results_common_pb2 import CreateResultsMetaDataRequest, CreateResultsMetaDataResponse, ListResultsRequest, ListResultsResponse
from ..protogen.common.results_filters_pb2 import Filters as rawFilters, FiltersAnd as rawFilterAnd, FilterField as rawFilterField, FilterStatus as rawFilterStatus
from ..protogen.common.results_fields_pb2 import ResultField
from ..common.filter import StringFilter, StatusFilter, DateFilter, NumberFilter, Filter
from ..protogen.common.sort_direction_pb2 import SortDirection
from ..common import Direction , Result
from ..protogen.common.results_fields_pb2 import ResultField, ResultRawField, ResultRawEnumField, RESULT_RAW_ENUM_FIELD_STATUS

class ResultFieldFilter:
    STATUS = StatusFilter(ResultField(result_raw_field=ResultRawField(field=RESULT_RAW_ENUM_FIELD_STATUS)), rawFilters, rawFilterAnd, rawFilterField, rawFilterStatus)

class ArmoniKResult:
    def __init__(self, grpc_channel: Channel):
        """ Result service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = ResultsStub(grpc_channel)

    def get_results_ids(self, session_id: str, names: List[str]) -> Dict[str, str]:
        return {r.name : r.result_id for r in cast(CreateResultsMetaDataResponse, self._client.CreateResultsMetaData(CreateResultsMetaDataRequest(results=[CreateResultsMetaDataRequest.ResultCreate(name = n) for n in names], session_id=session_id))).results}

    def list_results(self, result_filter: Filter, page: int = 0, page_size: int = 1000, sort_field: Filter = ResultFieldFilter.STATUS,sort_direction: SortDirection = Direction.ASC ) -> Tuple[int, List[Result]]:
        """List results based on a filter.

        Args:
            result_filter (Filter): Filter to apply when listing results
            page: page number to request, useful for pagination, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field to sort the resulting list by, defaults to the status
            sort_direction: direction of the sort, defaults to ascending
        Returns:
            A tuple containing :
            - The total number of results for the given filter
            - The obtained list of results
        """
        request: ListResultsRequest = ListResultsRequest(
            page=page,
            page_size=page_size,
            filters=cast(rawFilters, result_filter.to_disjunction().to_message()),
            sort=ListResultsRequest.Sort(field=cast(ResultField, sort_field.field), direction=sort_direction),
        )
        list_response: ListResultsResponse = self._client.ListResults(request)
        return list_response.total, [Result.from_message(r) for r in list_response.results]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: agent_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import agent_common_pb2 as agent__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x61gent_service.proto\x12\x19\x61rmonik.api.grpc.v1.agent\x1a\x12\x61gent_common.proto2\x8d\x07\n\x05\x41gent\x12h\n\nCreateTask\x12,.armonik.api.grpc.v1.agent.CreateTaskRequest\x1a*.armonik.api.grpc.v1.agent.CreateTaskReply(\x01\x12\x8c\x01\n\x15\x43reateResultsMetaData\x12\x37.armonik.api.grpc.v1.agent.CreateResultsMetaDataRequest\x1a\x38.armonik.api.grpc.v1.agent.CreateResultsMetaDataResponse\"\x00\x12t\n\rCreateResults\x12/.armonik.api.grpc.v1.agent.CreateResultsRequest\x1a\x30.armonik.api.grpc.v1.agent.CreateResultsResponse\"\x00\x12}\n\x10NotifyResultData\x12\x32.armonik.api.grpc.v1.agent.NotifyResultDataRequest\x1a\x33.armonik.api.grpc.v1.agent.NotifyResultDataResponse\"\x00\x12n\n\x0bSubmitTasks\x12-.armonik.api.grpc.v1.agent.SubmitTasksRequest\x1a..armonik.api.grpc.v1.agent.SubmitTasksResponse\"\x00\x12\x62\n\x0fGetResourceData\x12&.armonik.api.grpc.v1.agent.DataRequest\x1a\'.armonik.api.grpc.v1.agent.DataResponse\x12`\n\rGetCommonData\x12&.armonik.api.grpc.v1.agent.DataRequest\x1a\'.armonik.api.grpc.v1.agent.DataResponse\x12`\n\rGetDirectData\x12&.armonik.api.grpc.v1.agent.DataRequest\x1a\'.armonik.api.grpc.v1.agent.DataResponseB\x1c\xaa\x02\x19\x41rmoniK.Api.gRPC.V1.Agentb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'agent_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\031ArmoniK.Api.gRPC.V1.Agent'
  _globals['_AGENT']._serialized_start=71
  _globals['_AGENT']._serialized_end=980
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: partitions_common.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import partitions_fields_pb2 as partitions__fields__pb2
from . import partitions_filters_pb2 as partitions__filters__pb2
from . import sort_direction_pb2 as sort__direction__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17partitions_common.proto\x12\x1e\x61rmonik.api.grpc.v1.partitions\x1a\x17partitions_fields.proto\x1a\x18partitions_filters.proto\x1a\x14sort_direction.proto\"\xa8\x02\n\x0cPartitionRaw\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1c\n\x14parent_partition_ids\x18\x02 \x03(\t\x12\x14\n\x0cpod_reserved\x18\x03 \x01(\x03\x12\x0f\n\x07pod_max\x18\x04 \x01(\x03\x12]\n\x11pod_configuration\x18\x07 \x03(\x0b\x32\x42.armonik.api.grpc.v1.partitions.PartitionRaw.PodConfigurationEntry\x12\x1d\n\x15preemption_percentage\x18\x05 \x01(\x03\x12\x10\n\x08priority\x18\x06 \x01(\x03\x1a\x37\n\x15PodConfigurationEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xca\x02\n\x15ListPartitionsRequest\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x38\n\x07\x66ilters\x18\x03 \x01(\x0b\x32\'.armonik.api.grpc.v1.partitions.Filters\x12H\n\x04sort\x18\x04 \x01(\x0b\x32:.armonik.api.grpc.v1.partitions.ListPartitionsRequest.Sort\x1a\x8b\x01\n\x04Sort\x12=\n\x05\x66ield\x18\x01 \x01(\x0b\x32..armonik.api.grpc.v1.partitions.PartitionField\x12\x44\n\tdirection\x18\x02 \x01(\x0e\x32\x31.armonik.api.grpc.v1.sort_direction.SortDirection\"\x8a\x01\n\x16ListPartitionsResponse\x12@\n\npartitions\x18\x01 \x03(\x0b\x32,.armonik.api.grpc.v1.partitions.PartitionRaw\x12\x0c\n\x04page\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\r\n\x05total\x18\x04 \x01(\x05\"!\n\x13GetPartitionRequest\x12\n\n\x02id\x18\x01 \x01(\t\"W\n\x14GetPartitionResponse\x12?\n\tpartition\x18\x01 \x01(\x0b\x32,.armonik.api.grpc.v1.partitions.PartitionRawB!\xaa\x02\x1e\x41rmoniK.Api.gRPC.V1.Partitionsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'partitions_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002\036ArmoniK.Api.gRPC.V1.Partitions'
  _PARTITIONRAW_PODCONFIGURATIONENTRY._options = None
  _PARTITIONRAW_PODCONFIGURATIONENTRY._serialized_options = b'8\001'
  _globals['_PARTITIONRAW']._serialized_start=133
  _globals['_PARTITIONRAW']._serialized_end=429
  _globals['_PARTITIONRAW_PODCONFIGURATIONENTRY']._serialized_start=374
  _globals['_PARTITIONRAW_PODCONFIGURATIONENTRY']._serialized_end=429
  _globals['_LISTPARTITIONSREQUEST']._serialized_start=432
  _globals['_LISTPARTITIONSREQUEST']._serialized_end=762
  _globals['_LISTPARTITIONSREQUEST_SORT']._serialized_start=623
  _globals['_LISTPARTITIONSREQUEST_SORT']._serialized_end=762
  _globals['_LISTPARTITIONSRESPONSE']._serialized_start=765
  _globals['_LISTPARTITIONSRESPONSE']._serialized_end=903
  _globals['_GETPARTITIONREQUEST']._serialized_start=905
  _globals['_GETPARTITIONREQUEST']._serialized_end=938
  _globals['_GETPARTITIONRESPONSE']._serialized_start=940
  _globals['_GETPARTITIONRESPONSE']._serialized_end=1027
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: health_checks_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import health_checks_common_pb2 as health__checks__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bhealth_checks_service.proto\x12!armonik.api.grpc.v1.health_checks\x1a\x1ahealth_checks_common.proto2\x95\x01\n\x13HealthChecksService\x12~\n\x0b\x43heckHealth\x12\x35.armonik.api.grpc.v1.health_checks.CheckHealthRequest\x1a\x36.armonik.api.grpc.v1.health_checks.CheckHealthResponse\"\x00\x42#\xaa\x02 ArmoniK.Api.gRPC.V1.HealthChecksb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'health_checks_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\252\002 ArmoniK.Api.gRPC.V1.HealthChecks'
  _globals['_HEALTHCHECKSSERVICE']._serialized_start=95
  _globals['_HEALTHCHECKSSERVICE']._serialized_end=244
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from ..protogen.common.task_status_pb2 import TaskStatus as RawStatus, _TASKSTATUS, TASK_STATUS_CANCELLED, TASK_STATUS_CANCELLING, TASK_STATUS_COMPLETED, TASK_STATUS_CREATING, TASK_STATUS_DISPATCHED, TASK_STATUS_ERROR, TASK_STATUS_PROCESSED, TASK_STATUS_PROCESSING, TASK_STATUS_SUBMITTED, TASK_STATUS_TIMEOUT, TASK_STATUS_UNSPECIFIED, TASK_STATUS_RETRIED
from ..protogen.common.session_status_pb2 import SessionStatus as RawSessionStatus, _SESSIONSTATUS, SESSION_STATUS_UNSPECIFIED, SESSION_STATUS_CANCELLED, SESSION_STATUS_RUNNING
from ..protogen.common.result_status_pb2 import ResultStatus as RawResultStatus, _RESULTSTATUS, RESULT_STATUS_UNSPECIFIED, RESULT_STATUS_CREATED, RESULT_STATUS_COMPLETED, RESULT_STATUS_ABORTED, RESULT_STATUS_NOTFOUND
from ..protogen.common.worker_common_pb2 import HealthCheckReply
from ..protogen.common.sort_direction_pb2 import SORT_DIRECTION_ASC, SORT_DIRECTION_DESC

# This file is necessary because the grpc types aren't considered proper types


class HealthCheckStatus:
    NOT_SERVING = HealthCheckReply.NOT_SERVING
    SERVING = HealthCheckReply.SERVING
    UNKNOWN = HealthCheckReply.UNKNOWN


class TaskStatus:
    @staticmethod
    def name_from_value(status: RawStatus) -> str:
        return _TASKSTATUS.values_by_number[status].name

    CANCELLED = TASK_STATUS_CANCELLED
    CANCELLING = TASK_STATUS_CANCELLING
    COMPLETED = TASK_STATUS_COMPLETED
    CREATING = TASK_STATUS_CREATING
    DISPATCHED = TASK_STATUS_DISPATCHED
    ERROR = TASK_STATUS_ERROR
    PROCESSED = TASK_STATUS_PROCESSED
    PROCESSING = TASK_STATUS_PROCESSING
    SUBMITTED = TASK_STATUS_SUBMITTED
    RETRIED = TASK_STATUS_RETRIED
    TIMEOUT = TASK_STATUS_TIMEOUT
    UNSPECIFIED = TASK_STATUS_UNSPECIFIED


class Direction:
    ASC = SORT_DIRECTION_ASC
    DESC = SORT_DIRECTION_DESC


class SessionStatus:
    @staticmethod
    def name_from_value(status: RawSessionStatus) -> str:
        return _SESSIONSTATUS.values_by_number[status].name

    UNSPECIFIED = SESSION_STATUS_UNSPECIFIED
    RUNNING = SESSION_STATUS_RUNNING
    CANCELLED = SESSION_STATUS_CANCELLED

class ResultStatus:
    @staticmethod
    def name_from_value(status: RawResultStatus) -> str:
        return _RESULTSTATUS.values_by_number[status].name

    UNSPECIFIED = RESULT_STATUS_UNSPECIFIED
    CREATED = RESULT_STATUS_CREATED
    COMPLETED = RESULT_STATUS_COMPLETED
    ABORTED = RESULT_STATUS_ABORTED
    NOTFOUND = RESULT_STATUS_NOTFOUND
